package shape

sealed class Geometry

data class Point(val x: Double = Double.NaN, val y: Double = Double.NaN, val z: Double = Double.NaN) : Geometry()

data class LineString(val points: MutableList<Point> = mutableListOf()) : Geometry()

data class Polygon(val shell: LineString = LineString(), val holes: MutableList<LineString> = mutableListOf()) : Geometry()


sealed class GeometryCollection<T : Geometry>(val geometries: List<T> = emptyList()) : Geometry()

class MultiPoint(geometries: MutableList<Point>) : GeometryCollection<Point>(geometries)

class MultiLineString(geometries: MutableList<LineString>) : GeometryCollection<LineString>(geometries)

class MultiPolygon(geometries: MutableList<Polygon>) : GeometryCollection<Polygon>(geometries)


enum class ShapeType(val id: Int, val type: String)
{
    NULL(0, "null"),
    POINT(1, "point"),
    POLYLINE(3, "polyline"),
    POLYGON(5, "polygon"),
    MULTIPOINT(8, "multipoint"),
    POINTZ(11, "pointz"),
    POLYLINEZ(13, "polylinez"),
    POLYGONZ(15, "polygonz"),
    MULTIPOINTZ(18, "multipointz"),
    POINTM(21, "pointm"),
    POLYLINEM(23, "polylinem"),
    POLYGONM(25, "polygonm"),
    MULTIPOINTM(28, "multipointm"),
    MULTIPATCH(31, "multipatch")
}


data class BoundingBox(
    val minX: Double,
    val maxX: Double,
    val minY: Double,
    val maxY: Double,
    val minZ: Double,
    val maxZ: Double
)

data class Record(val recordNumber: Int, val contentLength: Int, val type: Int, val geometry: Geometry)

data class ShpHeader(val headerId: Int, val fileLength: Int, val version: Int, val shapeType: ShapeType)

data class Shp(val header: ShpHeader, val bb: BoundingBox, val records: List<Record>)
