package shape

import kotlinx.io.core.ByteReadPacket
import org.junit.Test
import java.io.File
import java.lang.IllegalStateException
import java.nio.file.Path
import java.nio.file.Paths
import kotlin.test.assertEquals

class ShpHeaderTest {

    private val dataDirectory = Paths.get(File(".").canonicalPath, "data")
    private val shpFile = dataDirectory.resolve("point.shp")

    @Test(expected = IllegalStateException::class)
    fun readPointShpHeader() {
        val packet = shpFile.toByteReadPacket()
        val shpHeader = readShpHeader(packet)
        assertEquals(shpHeader.shapeType, ShapeType.POINT)
    }

    @Test
    fun readPointShpHeaderCorrect() {
        val packet = shpFile.toByteReadPacket()
        val shpHeader = readShpHeaderCorrect(packet)
        assertEquals(shpHeader.shapeType, ShapeType.POINT)
    }

    private fun Path.toByteReadPacket(): ByteReadPacket =
        this.toFile().inputStream().use { inputStream -> ByteReadPacket(inputStream.readBytes()) }
}